import json
from dataclasses import dataclass


@dataclass(frozen=True)
class RawGroup:
    spn: str
    name: str
    kind: str
    direct_member_of: frozenset[str]
    member_of: frozenset[str]
    members: frozenset[str]
    scopes_map: frozenset[str]
    details: str


def is_filtered(k: RawGroup) -> bool:
    if k.name.startswith("vlan_"):
        return True
    if k.kind == "person" and "grp_active@sso.dgnum.eu" not in k.member_of:
        return True
    if k.name.startswith("idm_"):
        return True
    if k.name.startswith("ext_idm_provisioned_entities"):
        return True
    return False


groups = {}

direct_member_of = set()
member_of = set()
members = set()
scopes_map = set()
spn = None
name = None
kind = None
details = ""

with open("dgn_groups", "r") as f:
    lines = f.readlines()

for line in lines:
    sline = line.strip()
    if sline.startswith("---"):
        if spn is not None:
            if name is None:
                raise ValueError(f"Missing name of {spn}")
            if kind is None:
                raise ValueError(f"Missing kind of {spn}")
            groups[spn] = RawGroup(
                spn,
                name,
                kind,
                frozenset(direct_member_of),
                frozenset(member_of),
                frozenset(members),
                frozenset(scopes_map),
                details,
            )
        elif len(direct_member_of) > 0 or len(members) > 0:
            raise ValueError("Failed to parse groups, missing spn")
        direct_member_of = set()
        member_of = set()
        members = set()
        scopes_map = set()
        spn = None
        kind = None
        name = None
        details = ""
        continue
    elif sline.startswith("directmemberof: "):
        direct_member_of.add(sline[len("directmemberof: ") :])
    elif sline.startswith("memberof: "):
        member_of.add(sline[len("memberof: ") :])
    elif sline.startswith("member: "):
        members.add(sline[len("member: ") :])
    elif sline.startswith("oauth2_rs_scope_map: "):
        scopes_map.add(sline.split(":")[1].strip())
    elif sline.startswith("spn: "):
        spn = sline[len("spn: ") :]
    elif sline.startswith("name: "):
        name = sline[len("name: ") :]
    elif sline.startswith("class: group") and kind != "dyngroup":
        kind = "group"
    elif sline.startswith("class: dyngroup"):
        kind = "dyngroup"
    elif sline.startswith("class: person"):
        kind = "person"
    elif sline.startswith("class: service_account"):
        kind = "service"
    elif sline.startswith("class: oauth2_resource_server"):
        kind = "oauth2"
    details += sline + "\n"

cytodata = []

filtered_groups = set()

for k in groups:
    if is_filtered(groups[k]):
        continue
    filtered_groups.add(k)

for k in filtered_groups:
    color = "black"
    match groups[k].kind:
        case "group":
            color = "red"
        case "person":
            color = "green"
        case "dyngroup":
            color = "orange"
        case "service":
            color = "blue"
        case "oauth2":
            color = "cyan"
    cytodata.append(
        {
            "data": {
                "id": k,
                "label": k.removesuffix("@sso.dgnum.eu"),
                "color": color,
                "details": groups[k].details,
            }
        }
    )
    for p in groups[k].direct_member_of:
        if p not in filtered_groups:
            continue
        cytodata.append({"data": {"id": f"{k}-{p}", "source": k, "target": p}})
    for s in groups[k].scopes_map:
        if s not in filtered_groups:
            continue
        cytodata.append(
            {
                "data": {
                    "id": f"{s}-{k}",
                    "source": s,
                    "target": k,
                    "oauth_scope": True,
                }
            }
        )
    for m in groups[k].members:
        if m not in groups:
            cytodata.append(
                {
                    "data": {
                        "id": m,
                        "label": m.removesuffix("@sso.dgnum.eu"),
                        "color": "grey",
                        "details": "No details available",
                    }
                }
            )
            cytodata.append({"data": {"id": f"{m}-{k}", "source": m, "target": k}})

print(json.dumps(cytodata))
